; I2CProt - a MBus Protocol Mode Test & Example Program
; Version 0.99 (public beta) 14.1.1998
; Copyright (C) 1998 Osma Suominen <ozone@clinet.fi>
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; The file COPYING, which should have been included with this source file,
; contains the GNU General Public License, which applies to this program.
;
; Look in README.TXT for usage instructions.

#include "ti-85.h"

; Variables in graphics memory:

I2CAddress     = $8641
I2CInputBuffer = $8901
I2CInputLength = $8642
I2CCStart      = $8645                  ; Start counter
I2CCStop       = $8649                  ; Stop counter
I2CCMiss       = $8643                  ; Missed Start counter
I2CCTimeout    = $8647                  ; Timeout counter
I2CCChecksum   = $864B                  ; Checksum error counter

I2CSpeed       = $864D

I2COutputBuffer = $8981

I2CSlaveAddr   = $80DF                  ; Variable for temporary data

.org 0
.db "MBus Protocol Mode Tester",0

        ROM_CALL(CLEARLCD)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,PromptText
        add hl,de
        ROM_CALL(D_ZT_STR)

KeyLoop:
        halt                    ; Saves power
        call GET_KEY
        cp $37                  ; "EXIT"
        ret z
        cp $35                  ; "F1"
        jr z,Send
        cp $34                  ; "F2"
        JUMP_Z(Receive)

        jr KeyLoop

Send:
        ld a,0                  ; Own I2C address
        ld (I2CAddress),a

        ROM_CALL(CLEARLCD)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,SendText
        add hl,de
        ROM_CALL(D_ZT_STR)

        ld hl,I2CSlaveAddr
        ld (hl),64              ; Default slave addr = 64 = $40
        CALL_(GetHex)

        ld a,3
        CALL_(DispSpeed)

        ld hl,I2CSpeed
        ld (hl),4               ; Default speed = 4
        CALL_(GetHex)

        ld de,I2COutputBuffer
        ld hl,$1000
        ld bc, 125
        ldir                    ; Move 125 bytes of "random" data into buffer

        ld a,4
        CALL_(DispSending)

        ld a,(I2CSlaveAddr)
        ld d,a                  ; Slave address
        ld a,$ff                ; Protocol id
        ld hl,I2COutputBuffer   ; Start of data string
        ld c,0                  ; Error counter
        ld b,$80                ; No. of packets to send (128)
SendLoop:
        push bc
        ld b,125                ; Length of data
        CALL_(MI2C)
        pop bc
        push af
        ld a,c                  ; Count errors
        adc a,0
        ld c,a
        pop af
        djnz SendLoop

        ld a,4
        CALL_(DispError)

        ld h,0
        ld l,c
        ROM_CALL(D_HL_DECI)
        JUMP_(Exit)

DiscardData:
        xor a
        ld (I2CInputLength),a
        jr ReceiveLoop

Receive:
        ld a,64                 ; Own I2C address (emul. PCF8574)
        ld (I2CAddress),a

        ROM_CALL(CLEARLCD)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText
        add hl,de
        ROM_CALL(D_ZT_STR)
        
        CALL_(SI2CLoad)         ; Load interrupt routine
ReceiveLoop:
        ld a,(I2CInputLength)
        or a                    ; cp 0
        jr nz,DiscardData
        call GET_KEY
        cp $37
        jr nz,ReceiveLoop

        im 1                    ; Disable slave int routine

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText2

        add hl,de
        ROM_CALL(D_ZT_STR)
        ld hl,(I2CCStart)       ; Start counter
        ROM_CALL(D_HL_DECI)

        ld a,3
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText3
        add hl,de
        ROM_CALL(D_ZT_STR)
        ld hl,(I2CCStop)        ; Stop counter
        ROM_CALL(D_HL_DECI)

        ld a,4
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText4
        add hl,de
        ROM_CALL(D_ZT_STR)
        ld hl,(I2CCMiss)        ; Missed Start counter
        ROM_CALL(D_HL_DECI)

        ld a,5
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText5
        add hl,de
        ROM_CALL(D_ZT_STR)
        ld hl,(I2CCTimeout)     ; Timeout counter
        ROM_CALL(D_HL_DECI)

        ld a,6
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReceiveText6
        add hl,de
        ROM_CALL(D_ZT_STR)
        ld hl,(I2CCChecksum)    ; Checksum Error counter
        ROM_CALL(D_HL_DECI)

Exit:
        ld a,7
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ExitText
        add hl,de
        ROM_CALL(D_ZT_STR)

ExitLoop:
        halt
        call GET_KEY
        or a
        jr z,ExitLoop
        ret

PopExit:
        pop af
        jr Exit

GetHex:
        ld b,2                  ; Show 2 hex digits
        xor a
HexLoop:
        rld
        push af
        add a,'0'
        cp '9' + 1
        jr c,Number
        add a,'A' - '0' - 10
Number: ROM_CALL(TX_CHARPUT)
        pop af
        djnz HexLoop
        rld

HexKeyLoop:
        push hl
        halt
        call GET_KEY
        pop hl
        cp $9                   ; "Enter"
        ret z                   ; Value entered
        cp $37                  ; "Exit"
        jr z,PopExit
        cp $0e                  ; "E"
        jr c,HexKeyLoop
        cp $2F                  ; "A"+1
        jr nc,HexKeyLoop
        sub $0e
        push hl
        ld hl,(PROGRAM_ADDR)
        ld de,Scan2HexTable
        add hl,de
        ld d,0
        ld e,a
        add hl,de
        ld a,(hl)
        pop hl
        cp $10
        jr z,HexKeyLoop         ; Loop if invalid key 
        rld
        ld a,(CURSOR_COL)       ; Go left 2 chars
        dec a
        dec a
        ld (CURSOR_COL),a
        jr GetHex

DispSpeed:
        ld (CURSOR_ROW),a
        ld a,4
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,SpeedText
        add hl,de
        ROM_CALL(D_ZT_STR)
        ret

DispSending:
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,SendingText
        add hl,de
        ROM_CALL(D_ZT_STR)
        ret

DispError:
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,ErrorText
        add hl,de
        ROM_CALL(D_ZT_STR)
        ret

PromptText:
.db " Enter mode:         "
.db " F1-Send F2-Receive  ",0

SendText:
.db "Sending 128 packets, "
.db "128 bytes in each... "
.db "Dest addr: ",0

SpeedText:
.db "Speed: ",0

SendingText:
.db "Sending...",0

ErrorText:
.db "Errors ",0

ReceiveText:
.db "Receiving packets... "
.db "Press EXIT to stop.  ",0

ReceiveText2:
.db "Starts:         ",0

ReceiveText3:
.db "Stops:          ",0

ReceiveText4:        
.db "Missed Starts:  ",0

ReceiveText5:
.db "Timeouts:       ",0

ReceiveText6:
.db "Checksum Errors:",0

ExitText:
.db "Press any key",0

Scan2HexTable:
; Beginning from scancode $e which happens to be hex digit $e
; $10 means invalid key
.db $e,$10,$10,$10,$3,$6,$9,$10,$d,$10,$10,$10,$2,$5,$8,$10
.db $c,$10,$10,$0,$1,$4,$7,$10,$b,$10,$10,$10,$10,$10,$10,$f,$a

; Slave interrupt routines are not loaded when the master routines are
; invoked, so SIMULT_SLAVE is undefined.
; #define SIMULT_SLAVE

#define NO_RAW

#include "master.asm"
#include "slave.asm"

.end
