; I2CRaw - a MBus Raw Mode Test & Example Program
; Version 0.99 (public beta) 14.1.1998
; Copyright (C) 1998 Osma Suominen <ozone@clinet.fi>
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

; Look in README.TXT for usage instructions.

#include "ti-85.h"

; Variables in text memory:



I2CSlaveAddr    = $80DF                 ; Variable for temporary data
I2CBuffer       = $80E0                 ; Buffer for read mode input

.org 0
.db "MBus Raw Mode Tester",0

        ROM_CALL(CLEARLCD)
        CALL_(MI2CLoad)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,PromptText
        add hl,de
        ROM_CALL(D_ZT_STR)

KeyLoop:
        halt                    ; Saves power
        call GET_KEY
        cp $37                  ; "EXIT"
        ret z
        cp $35                  ; "F1"
        jr z,Write
        cp $34                  ; "F2"
        jr z,Read

        jr KeyLoop

Write:
        ROM_CALL(CLEARLCD)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,WriteText
        add hl,de
        ROM_CALL(D_ZT_STR)

        ld hl,I2CSlaveAddr
        ld (hl),64              ; Default destination
        CALL_(GetHex)

        ld a,4
        CALL_(DispSending)

        ld a,(I2CSlaveAddr)     ; Slave addr
        and $fe                 ; 11111110 - make it a write address
        ld d,a
        ld bc,0                 ; Data length
        ld a,0                  ; Protocol id
        ld hl,0                 ; Start of address space

        CALL_(MI2C)

        JUMP_NC(Exit)
        ld a,5
        CALL_(DispError)
        
        JUMP_(Exit)

Read:
        ROM_CALL(CLEARLCD)
        xor a                   ; = ld a,0
        ld (CURSOR_ROW),a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ReadText
        add hl,de
        ROM_CALL(D_ZT_STR)

        ld hl,I2CSlaveAddr
        ld (hl),65              ; Default slave addr = $41 (PCF8574)
        CALL_(GetHex)

        ld a,(hl)
        or $01                  ; 00000001 - make it a read address
        ld d,a                  ; Slave addr
        ld bc,4                 ; Data length
        ld a,0                  ; Protocol id
        ld hl,I2CBuffer

        CALL_(MI2C)
        jr c, ReadError

        ld ix,CURSOR_ROW
        ld b,4                  ; Bytes to show
        ld hl,I2CBuffer         ; First byte

ReadLoop:
        inc (ix+0)
        xor a
        ld (CURSOR_COL),a
        push hl
        ld l,(hl)
        ld h,a
        ROM_CALL(D_HL_DECI)
        pop hl
        inc hl
        djnz ReadLoop
        
        jr Exit

ReadError:
        ld a,3
        CALL_(DispError)

Exit:
        ld a,7
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a

        ld hl,(PROGRAM_ADDR)
        ld de,ExitText
        add hl,de
        ROM_CALL(D_ZT_STR)

ExitLoop:
        halt
        call GET_KEY
        or a
        jr z,ExitLoop
        ret

PopExit:
        pop af
        jr Exit

GetHex:
        ld b,2                  ; Show 2 hex digits
        xor a
HexLoop:
        rld
        push af
        add a,'0'
        cp '9' + 1
        jr c,Number
        add a,'A' - '0' - 10
Number: ROM_CALL(TX_CHARPUT)
        pop af
        djnz HexLoop
        rld

HexKeyLoop:
        push hl
        halt
        call GET_KEY
        pop hl
        cp $9                   ; "Enter"
        ret z                   ; Value entered
        cp $37                  ; "Exit"
        jr z,PopExit
        cp $0e                  ; "E"
        jr c,HexKeyLoop
        cp $2F                  ; "A"+1
        jr nc,HexKeyLoop
        sub $0e
        push hl
        ld hl,(PROGRAM_ADDR)
        ld de,Scan2HexTable
        add hl,de
        ld d,0
        ld e,a
        add hl,de
        ld a,(hl)
        pop hl
        cp $10
        jr z,HexKeyLoop         ; Loop if invalid key 
        rld
        ld a,(CURSOR_COL)       ; Go left 2 chars
        dec a
        dec a
        ld (CURSOR_COL),a
        jr GetHex

DispSending:
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,SendingText
        add hl,de
        ROM_CALL(D_ZT_STR)
        ret

DispError:
        ld (CURSOR_ROW),a
        xor a
        ld (CURSOR_COL),a
        ld hl,(PROGRAM_ADDR)
        ld de,ErrorText
        add hl,de
        ROM_CALL(D_ZT_STR)
        ret

PromptText:
.db " Enter mode: (raw)   "
.db " F1-Write F2-Read",0

SendingText:
.db "Sending...",0

ErrorText:
.db "Errors ",0

WriteText:
.db "Writing 64kBytes in a"
.db "single block...      "
.db "Dest addr: ",0

ReadText:
.db "Reading 4 Bytes in a "
.db "single block...      "
.db "Source addr: ",0

ExitText:
.db "Press any key",0

Scan2HexTable:
; Beginning from scancode $e which happens to be hex digit $e
; $10 means invalid key
.db $e,$10,$10,$10,$3,$6,$9,$10,$d,$10,$10,$10,$2,$5,$8,$10
.db $c,$10,$10,$0,$1,$4,$7,$10,$b,$10,$10,$10,$10,$10,$10,$f,$a

#define ZERO_WAIT_STATE
#define ONLY_RAW
; #define NO_SYNCH
#define USE_TM2
; #define 10_BIT_ADDR

#include "master.asm"

.end
